<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\Util\Functions;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\RelatoriosRNFService;
use Teknisa\MAN\Utilities;
use \Exception as Exception;

class RelatoriosRNFController {
    protected $relatoriosRNFService;
    protected $utilService;

    public function __construct(RelatoriosRNFService $relatoriosRNFService, Utilities\Util $utilService) {
        $this->relatoriosRNFService = $relatoriosRNFService;
        $this->utilService = $utilService;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $result = [];
        
        $conditions = $this->utilService->getParamsAsArray($request);
        $conditions = $this->buildParams($conditions);
        
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])) {
            $result = $this->relatoriosRNFService->find($conditions);

            $response->addDataSet(new DataSet("sdx35000_relatoriosRNF", $result));
        }
        
        return $result;
    }

    private function buildParams($conditions) {
        $result = [];
        
        if(!Functions::arrayKeyExists('CDFILIAL', $conditions)) {
            $result['P_CDFILIAL'] = 'T';
        } else {
            $result['P_CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $result['CDFILIAL'] = $conditions['CDFILIAL'];
        }
        
        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $result['P_DTINICIAL'] = $conditions['DTENTRVENDA'][0];
            $result['P_DTFINAL']   = $conditions['DTENTRVENDA'][1];
        }
        
        if(Functions::arrayKeyExists('STATUS', $conditions)) {
            $result['P_IDSTATUS'] = implode('_', $conditions['STATUS']);
        } else {
            $result['P_IDSTATUS'] = 'P_I_R';
        }
        
        if(Functions::arrayKeyExists('IDRELATORIOFIELD', $conditions)) {
            $result['IDRELATORIO'] = $conditions['IDRELATORIOFIELD'];
        } else {
            $result['IDRELATORIO'] = 0;
        }
        
        if(Functions::arrayKeyExists('CDOPERADOR', $conditions)) {
            $result['P_CDOPERADOR'] = $conditions['CDOPERADOR'];
        }
        
        if($result['IDRELATORIO'] != 0) {
            unset($result['CDFILIAL']);
        }
        
        return $result;
    }
}