<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use \Zeedhi\Framework\Util\Functions;

class RelEntregasPorMotoboyService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function fetchReport($conditions){
        $params = self::buildParams($conditions);
        
        if($conditions["TIPORELATORIO"] == "A") {
            return $this->entityManager->getConnection()->fetchAll(self::GET_ENTREGAS_POR_MOTOBOY_QRY, $params);
        }
        else {
            return $this->entityManager->getConnection()->fetchAll(self::GET_ENTREGAS_POR_MOTOBOY_QRY_DIA, $params);
        }
    }
    
    private function buildParams($conditions){
        if(Functions::arrayKeyExists('CDFILIAL', $conditions)){
            $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
            $conditions['TCDFILIAL'] = NULL;
        } else {
            $conditions['CDFILIAL'] = NULL;
            $conditions['TCDFILIAL'] = 'T';
        }
        
        if(Functions::arrayKeyExists('DTENTRVENDA', $conditions)){
            $conditions['DTINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DTFIN'] = $conditions['DTENTRVENDA'][1];
        }
        
        if(!Functions::arrayKeyExists('VRTAXAMOTO', $conditions)){
            $conditions['VRTAXAMOTO'] = 100;
        }
        
        $params = array(
            ':CDFILIAL'   => $conditions['CDFILIAL'],
            ':TCDFILIAL'  => $conditions['TCDFILIAL'],
            ':DTINI'      => $conditions['DTINI'], 
            ':DTFIN'      => $conditions['DTFIN'], 
            ':NRORG'      => $conditions['NRORG'],
            ':VRTAXAMOTO' => $conditions['VRTAXAMOTO']
        );
        
        return $params;
    }
    
    const GET_ENTREGAS_POR_MOTOBOY_QRY = "
        SELECT V.CDFILIAL, F.NMFILIAL, V.CDCAIXA, V.NRSEQVENDA, NVL(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS NRCUPOMFIS,V.DTVENDA,V.CDVENDEDOR, E.NMRAZSOCVEN,
          V.DSENDECONSVENDA, V.CDBAIRRO, NVL(V.DSBAIRRO,B.NMBAIRRO) AS NMBAIRRO, ENTG.NRENTREGAS, '1' AS QTDE, C.NMCONSUMIDOR, CAVR.VRVENDA , V.VRTXSEVENDA, ((:VRTAXAMOTO/100)* V.VRTXSEVENDA) VRTAXAMOTO2,EN.NRCELULARVEN, E.DSPLACAVEICULO,
          E.VRDIARIAVENDOR, V.CDFILIAL||' '||F.NMFILIAL AS LABELFILIAL, V.CDVENDEDOR||' '||E.NMRAZSOCVEN AS LABELMOTOQUEIRO
        FROM VENDA V 
         JOIN VENDEDOR E
           ON V.CDVENDEDOR   = E.CDVENDEDOR
            AND V.NRORG = E.NRORG
         LEFT JOIN BAIRRO B
          ON V.CDPAIS = B.CDPAIS
           AND V.SGESTADO = B.SGESTADO
           AND V.CDMUNICIPIO  = B.CDMUNICIPIO
           AND V.CDBAIRRO  = B.CDBAIRRO
           AND V.NRORG = B.NRORG
         JOIN FILIAL F
          ON V.CDFILIAL = F.CDFILIAL
           AND V.NRORG = F.NRORG
         JOIN CONSUMIDOR C 
          ON  V.CDCONSUMIDOR = C.CDCONSUMIDOR
          AND V.CDCLIENTE = C.CDCLIENTE 
          AND V.NRORG = C.NRORG
         LEFT JOIN ENDEVENDE EN
          ON V.CDVENDEDOR = EN.CDVENDEDOR
          AND V.NRORG = EN.NRORG
         JOIN (SELECT V.CDFILIAL, V.CDVENDEDOR, COUNT(V.CDFILIAL||V.CDCAIXA||V.NRSEQVENDA) AS NRENTREGAS, V.NRORG
               FROM VENDA V
               JOIN FILIAL F
                ON V.CDFILIAL = F.CDFILIAL 
                 AND V.NRORG = F.NRORG
               WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                 AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                 AND V.NRENTREGAPEDVND IS NOT NULL
                 AND V.IDSITUVENDA = 'O'
               GROUP BY V.CDFILIAL, V.CDVENDEDOR, V.NRORG
              ) ENTG
          ON V.CDFILIAL = ENTG.CDFILIAL
           AND V.CDVENDEDOR   = ENTG.CDVENDEDOR
           AND V.NRORG = ENTG.NRORG
         JOIN ( SELECT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, SUM(ITV.QTPRODVEND * ITV.VRUNITVEND - ITV.VRDESITVEND + ITV.VRACRITVEND) AS VRVENDA, V.NRORG
                FROM VENDA V 
                JOIN ITEMVENDA ITV
                 ON  V.CDFILIAL    = ITV.CDFILIAL
                  AND V.CDCAIXA     = ITV.CDCAIXA
                  AND V.NRSEQVENDA  = ITV.NRSEQVENDA
                  AND V.NRORG  = ITV.NRORG
                JOIN FILIAL F
                 ON V.CDFILIAL = F.CDFILIAL 
                 AND V.NRORG = F.NRORG
               WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                 AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                 AND V.NRENTREGAPEDVND IS NOT NULL    
                 AND V.IDSITUVENDA = 'O'      
              GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.NRORG
              ) CAVR
          ON V.CDFILIAL = CAVR.CDFILIAL
           AND V.CDCAIXA = CAVR.CDCAIXA
           AND V.NRSEQVENDA = CAVR.NRSEQVENDA
           AND V.NRORG = CAVR.NRORG
        WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
          AND V.IDSITUVENDA  = 'O'
          AND V.NRENTREGAPEDVND IS NOT NULL
          AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
          AND V.NRORG = :NRORG
        ORDER BY 
           V.CDFILIAL, V.CDVENDEDOR, V.DTVENDA
    ";
    
    const GET_ENTREGAS_POR_MOTOBOY_QRY_DIA = "
        SELECT CONS.CDFILIAL, CONS.NMFILIAL, CONS.CDCAIXA, CONS.DTVENDA, CONS.CDVENDEDOR, CONS.NMRAZSOCVEN, SUM(CONS.QTDE) AS QTDE, SUM(CONS.VRVENDA) AS VRVENDA, SUM(CONS.VRTXSEVENDA) AS VRTXSEVENDA, SUM(CONS.VRTAXAMOTO2) AS VRTAXAMOTO2, CONS.LABELFILIAL, CONS.LABELMOTOQUEIRO
         FROM (SELECT V.CDFILIAL, F.NMFILIAL, V.CDCAIXA, V.NRSEQVENDA, NVL(V.NRNOTAFISCALCE, V.NRCUPOMFIS) AS NRCUPOMFIS, TO_CHAR(v.dtvenda,'DD/MM/YYYY') AS dtvenda, V.CDVENDEDOR, E.NMRAZSOCVEN,
                   V.DSENDECONSVENDA, V.CDBAIRRO, NVL(V.DSBAIRRO,B.NMBAIRRO) AS NMBAIRRO, ENTG.NRENTREGAS, '1' AS QTDE, C.NMCONSUMIDOR, CAVR.VRVENDA , V.VRTXSEVENDA, ((:VRTAXAMOTO/100)* V.VRTXSEVENDA) VRTAXAMOTO2,EN.NRCELULARVEN, E.DSPLACAVEICULO,
                   E.VRDIARIAVENDOR, V.CDFILIAL||' '||F.NMFILIAL AS LABELFILIAL, V.CDVENDEDOR||' '||E.NMRAZSOCVEN AS LABELMOTOQUEIRO
                 FROM VENDA V 
                  JOIN VENDEDOR E
                    ON V.CDVENDEDOR   = E.CDVENDEDOR
                     AND V.NRORG = E.NRORG
                  LEFT JOIN BAIRRO B
                   ON V.CDPAIS = B.CDPAIS
                    AND V.SGESTADO = B.SGESTADO
                    AND V.CDMUNICIPIO  = B.CDMUNICIPIO
                    AND V.CDBAIRRO  = B.CDBAIRRO
                    AND V.NRORG = B.NRORG
                  JOIN FILIAL F
                   ON V.CDFILIAL = F.CDFILIAL
                    AND V.NRORG = F.NRORG
                  JOIN CONSUMIDOR C 
                   ON  V.CDCONSUMIDOR = C.CDCONSUMIDOR
                   AND V.CDCLIENTE = C.CDCLIENTE 
                   AND V.NRORG = C.NRORG
                  LEFT JOIN ENDEVENDE EN
                   ON V.CDVENDEDOR = EN.CDVENDEDOR
                   AND V.NRORG = EN.NRORG
                  JOIN (SELECT V.CDFILIAL, V.CDVENDEDOR, COUNT(V.CDFILIAL||V.CDCAIXA||V.NRSEQVENDA) AS NRENTREGAS, V.NRORG
                        FROM VENDA V
                        JOIN FILIAL F
                         ON V.CDFILIAL = F.CDFILIAL 
                          AND V.NRORG = F.NRORG
                        WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                          AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                          AND V.NRENTREGAPEDVND IS NOT NULL
                          AND V.IDSITUVENDA = 'O'
                        GROUP BY V.CDFILIAL, V.CDVENDEDOR, V.NRORG
                       ) ENTG
                   ON V.CDFILIAL = ENTG.CDFILIAL
                    AND V.CDVENDEDOR   = ENTG.CDVENDEDOR
                    AND V.NRORG = ENTG.NRORG
                  JOIN ( SELECT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, SUM(ITV.QTPRODVEND * ITV.VRUNITVEND - ITV.VRDESITVEND + ITV.VRACRITVEND) AS VRVENDA, V.NRORG
                         FROM VENDA V 
                         JOIN ITEMVENDA ITV
                          ON  V.CDFILIAL    = ITV.CDFILIAL
                           AND V.CDCAIXA     = ITV.CDCAIXA
                           AND V.NRSEQVENDA  = ITV.NRSEQVENDA
                           AND V.NRORG  = ITV.NRORG
                         JOIN FILIAL F
                          ON V.CDFILIAL = F.CDFILIAL 
                          AND V.NRORG = F.NRORG
                        WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                          AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                          AND V.NRENTREGAPEDVND IS NOT NULL    
                          AND V.IDSITUVENDA = 'O'      
                       GROUP BY V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.NRORG
                       ) CAVR
                   ON V.CDFILIAL = CAVR.CDFILIAL
                    AND V.CDCAIXA = CAVR.CDCAIXA
                    AND V.NRSEQVENDA = CAVR.NRSEQVENDA
                    AND V.NRORG = CAVR.NRORG
                 WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                   AND V.IDSITUVENDA  = 'O'
                   AND V.NRENTREGAPEDVND IS NOT NULL
                   AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN
                   AND V.NRORG = :NRORG
                 ORDER BY 
                    V.CDFILIAL, V.CDVENDEDOR, V.DTVENDA) CONS
         GROUP BY CONS.CDFILIAL, CONS.NMFILIAL, CONS.CDCAIXA, CONS.DTVENDA, CONS.CDVENDEDOR, CONS.NMRAZSOCVEN, CONS.LABELFILIAL, CONS.LABELMOTOQUEIRO
         ORDER BY CONS.CDFILIAL, CONS.CDVENDEDOR, CONS.DTVENDA
    ";
}