<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Factories\RelatoriosRNFFactory;

class RelatoriosRNFService {
    protected $relatoriosRNFFactory;
    protected $inOrdem;
    protected $cdsRelatorio;
    protected $stNovoItem;

    public function __construct(RelatoriosRNFFactory $relatoriosRNFFactory) {
        $this->relatoriosRNFFactory = $relatoriosRNFFactory;
    }

    public function find($params) {
        $result = [];
        
        if($params['IDRELATORIO'] == 0) {
            // Relatório Validações RNF
            foreach($params['CDFILIAL'] as $cdfilial) {
                array_push($result, $this->relatoriosRNFFactory->findValidacoesRNF($params, $cdfilial));
            }
            
            $result = $this->formataDados($result);
            
        } else if($params['IDRELATORIO'] == 1) {
            // Relatório Resumo de Notas Fiscais
            $result = $this->relatoriosRNFFactory->findResumoNotasFiscais($params);
            
        } else if($params['IDRELATORIO'] == 2) {
            // Relatório Detalhamento de RNF
            $result = $this->relatoriosRNFFactory->findDetalhamentoRNF($params);
            
        } else if($params['IDRELATORIO'] == 3) {
            // Relatório Validações Baseadas em RNF
            $result = $this->getRelVdRNF($params);
        }
        
        return $result;
    }
    
    private function formataDados($array) {
        $result = [];
        
        foreach($array as $i => $arrayExterno) {
            foreach($arrayExterno as $j => $arrayInterno) {
                if(isset($arrayExterno)) {
                    $result[]['DSVALIDACAO'] = $arrayInterno['DSVALIDACAO'];
                }
            }
        }
        
        return $result;
    }

    private function getRelVdRNF($params) {
        try {
            $this->inOrdem = 0;

            // roda query
            $adqRelFil = $this->relatoriosRNFFactory->adqRelFil($params);

            if(!empty($adqRelFil)) {
                foreach($adqRelFil as $relFil) {
                    $this->validacaoGeral($relFil);

                    // roda query
                    $adqRelBody = $this->relatoriosRNFFactory->adqRelBody($params, $relFil['CDFILIAL']);

                    if(!empty($adqRelBody)) {
                        foreach($adqRelBody as $relBody) {
                            // roda query
                            $adqRelItens = $this->relatoriosRNFFactory->adqRelItens($relFil, $relBody);

                            if(!empty($adqRelItens)) {
                                foreach($adqRelItens as $relItens) {
                                    $this->stNovoItem = 'S';
                                    $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();

                                    $this->infItem($relBody, $relFil, $relItens);
                                    
                                    // roda query
                                    $adqCodExtProd = $this->relatoriosRNFFactory->adqCodExtProd($relItens['CDARVPROD']);

                                    if(!emtpy($adqCodExtProd)) {
                                        $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens);

                                        $this->cdsRelatorio[]['DSVALIDACAO'] = $adqCodExtProd['VDPRODUTO'];
                                    }

                                    // roda query
                                    $adqRelNCM = $this->relatoriosRNFFactory->adqRelNCM($relItens['CDARVPROD']);

                                    if(!empty($adqRelNCM)) {
                                        $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens);

                                        $this->cdsRelatorio[]['DSVALIDACAO'] = $adqRelNCM['DSVALIDACAO'];
                                    }

                                    // roda query
                                    $adqTamNCM = $this->relatoriosRNFFactory->adqTamNCM($relItens['CDARVPROD']);

                                    if(!empty($adqTamNCM)) {
                                        $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();

                                        $this->infItem($relBody, $relFil, $relItens);

                                        $this->cdsRelatorio[]['DSVALIDACAO'] = $adqTamNCM['DSVALIDACAO'];
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $result = $this->cdsRelatorio;
            return $result;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    private function validacaoGeral($relFil, $params) {
        $this->checaPeriodo($relFil, $params);
        $this->checaCodExtFil($relFil);
        $this->checaTamCodFil($relFil);
        $this->checaCodExtEmpr($relFil);
        $this->checaCodConvFilial($relFil);
        $this->checaClienPadrao($relFil);
        $this->checaCodExtClienPadrao($relFil);
        $this->checaTipoEmisCaixa($relFil);
        $this->checaTamCodExtEmpr($relFil);
        $this->checaUltFechamento($relFil, $params);
    }

    private function infItem($adqRelBody, $adqRelFil, $adqRelItens) {
        $this->infFilial($adqRelFil);

        $this->cdsRelatorio[]['NOTA']    = $adqRelBody['NOTA'];
        $this->cdsRelatorio[]['CDSERIE'] = $adqRelBody['CDSERIE'];
        $this->cdsRelatorio[]['DTNOTA']  = $adqRelBody['DTNOTA'];
        $this->cdsRelatorio[]['CPFCNPJ'] = $adqRelFil['CPFCNPJ'];
        $this->cdsRelatorio[]['CDITEM']  = $adqRelItens['CDITEM'];
        $this->cdsRelatorio[]['NMITEM']  = $adqRelItens['NMITEM'];

        if(!empty($adqRelItens)) {
            $this->cdsRelatorio[]['DSNCM'] = substr($adqRelItens['CDNCM'], 1, 4) . '.' . 
                                            substr($adqRelItens['CDNCM'], 5, 2) . '.' . 
                                            substr($adqRelItens['CDNCM'], 7, 2);
        }

        $this->cdsRelatorio[]['VRPRODUTOS'] = $adqRelItens['QTDEITEM'] * $adqRelItens['VRUNIT'];
        $this->cdsRelatorio[]['VROPERACAO'] = $adqRelItens['VRITEM'];
        $this->cdsRelatorio[]['TIPOOPERAC'] = $adqRelBody['CODOPER'];

        // roda query
        $adqRelImposto = $this->relatoriosRNFFactory->adqRelImposto($adqRelBody, $adqRelFil, $adqRelItens);

        if(!empty($adqRelImposto)) {
            $this->cdsRelatorio[]['VLRICMS']   = $adqRelImposto['VRIMPOPRODIT'];
            $this->cdsRelatorio[]['VLRPIS']    = $adqRelImposto['VRIMPPIS'];
            $this->cdsRelatorio[]['VLRCOFINS'] = $adqRelImposto['VRIMPCOFINS'];
            $this->cdsRelatorio[]['CDCFOP']    = $adqRelImposto['CDCFOPPROD'];
        }

        // roda query
        $adqRelTipoDoc = $this->relatoriosRNFFactory->adqRelTipoDoc($adqRelItens);

        if(empty($adqRelTipoDoc)) {
            $this->cdsRelatorio[]['DSTIPODOC'] = "";

            if($this->stNovoItem == 'S') {
                $this->stNovoItem = 'N';
                $this->cdsRelatorio[]['DSVALIDACAO'] = "TIPO DE DOCUMENTO NAO CADASTRADO PARA O CFOP " . $adqRelItens['CDCFOPPROD'] . 
                                                     ", NCM " . $adqRelItens['CDNCM'] . " E CST (ICMS) " . $adqRelItens['CDCSTPRODI'];
            }
        } else {
            $this->cdsRelatorio[]['DSTIPODOC'] = $adqRelTipoDoc['CDTPDOC'];
        }
    }

    private function seqOrdem() {
        inc($this->inOrdem);
        return $this->inOrdem;
    }

    private function infFilial($adqRelFil) {
        $this->cdsRelatorio[]['CDFILIAL']     = $adqRelFil['CDFILIALEXT'];
        $this->cdsRelatorio[]['NMFILIAL']     = $adqRelFil['NMFILIAL'];
        $this->cdsRelatorio[]['CDCLIENTE']    = $adqRelFil['CDCLIENTE'];
        $this->cdsRelatorio[]['NMRAZSOCCLIE'] = $adqRelFil['NMRAZSOCCLIE'];
    }

    private function checaPeriodo($relFil, $params) {
        // roda query
        $adqRelPeriodo = $this->relatoriosRNFFactory->adqRelPeriodo($relFil, $params);

        if(!empty($adqRelPeriodo)) {
            foreach($adqRelPeriodo as $relPeriodo) {
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relPeriodo['DSVALIDACAO'];
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtFil($relFil) {
        // roda query
        $adqRelCodExtFil = $this->relatoriosRNFFactory->adqRelCodExtFil($relFil);

        if(!empty($adqRelCodExtFil)) {
            foreach($adqRelCodExtFil as $relCodExtFil) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relCodExtFil['FILIALVD'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTamCodFil($relFil) {
        // roda query
        $adqRelTamCodFil = $this->relatoriosRNFFactory->adqRelTamCodFil($relFil);

        if(!empty($adqRelTamCodFil)) {
            foreach($adqRelTamCodFil as $relTamCodFil) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relTamCodFil['FILIALVD'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtEmpr($relFil) {
        // roda query
        $adqRelCodExtEmpr = $this->relatoriosRNFFactory->adqRelCodExtEmpr($relFil);

        if(!empty($adqRelCodExtEmpr)) {
            foreach($adqRelCodExtEmpr as $relCodExtEmpr) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relCodExtEmpr['DSVALIDACAO'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodConvFilial($relFil) {
        // roda query
        $adqRelConvFil = $this->relatoriosRNFFactory->adqRelConvFil($relFil);

        if(!empty($adqRelConvFil)) {
            foreach($adqRelConvFil as $relConvFil) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relConvFil['VDFILIAL'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaClienPadrao($relFil) {
        // roda query
        $adqRelClienPadrao = $this->relatoriosRNFFactory->adqRelClienPadrao($relFil);

        if(!empty($adqRelClienPadrao)) {
            foreach($adqRelClienPadrao as $relClienPadrao) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relClienPadrao['VDFILIAL'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaCodExtClienPadrao($relFil) {
        // roda query
        $adqRelCodExtClien = $this->relatoriosRNFFactory->adqRelCodExtClien($relFil);

        if(!empty($adqRelCodExtClien)) {
            foreach($adqRelCodExtClien as $relCodExtClien) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relCodExtClien['VDFILIAL'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTipoEmisCaixa($relFil) {
        // roda query
        $adqRelTipoEmis = $this->relatoriosRNFFactory->adqRelTipoEmis($relFil);

        if(!empty($adqRelTipoEmis)) {
            foreach($adqRelTipoEmis as $relTipoEmis) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relTipoEmis['VDFILIAL'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaTamCodExtEmpr($relFil) {
        // roda query
        $adqRelTamCodExtEmpr = $this->relatoriosRNFFactory->adqRelTamCodExtEmpr($relFil);

        if(!empty($adqRelTamCodExtEmpr)) {
            foreach($adqRelTamCodExtEmpr as $relTamCodExtEmpr) {
                $this->infFilial($relFil);
                $this->cdsRelatorio[]['DSVALIDACAO'] = $relTamCodExtEmpr['VDEMPRESA'];                
                $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
            }
        }
    }

    private function checaUltFechamento($relFil, $params) {
        // roda query
        $adqMaxDtAbert = $this->relatoriosRNFFactory->adqMaxDtAbert($relFil, $params);

        if(!empty($adqMaxDtAbert)) {
            foreach($adqMaxDtAbert as $maxDtAbert) {
                // roda query
                $adqChecaFechamento = $this->relatoriosRNFFactory->adqChecaFechamento($maxDtAbert);

                if(!empty($adqChecaFechamento)) {
                    $this->infFilial($relFil);
                    $this->cdsRelatorio[]['DSVALIDACAO'] = $adqChecaFechamento['DSVALIDACAO'];                
                    $this->cdsRelatorio[]['NRORDEM'] = $this->seqOrdem();
                }
            }
        }
    }
}